
# q-expansion of newform 6027.2.a.s, downloaded from the LMFDB on 06 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 6027
weight = 2
poly_data = [2, -4, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0], 1], [[0, 1, 0], 1], [[-3, 0, 1], 1]]

hecke_ring_character_values = None
aps_data = [[0, 1, 0], [1, 0, 0], [-1, -1, 1], [0, 0, 0], [-1, -1, 0], [-3, 1, -1], [-1, 1, -2], [-1, 1, -1], [-3, -1, 1], [-1, -3, 0], [2, -4, -1], [6, 2, -1], [-1, 0, 0], [4, -2, 5], [-1, -1, -2], [6, -4, 2], [2, 2, 2], [2, 2, -1], [2, 6, -4], [-11, 1, 0], [-2, 2, 3], [-8, 4, -2], [5, -1, 3], [-6, 4, 0], [3, 3, -1], [7, -3, -2], [-4, -2, 3], [0, 8, -2], [3, -1, 1], [-7, 5, -5], [2, -6, 0], [-3, -5, 1], [-9, 5, 2], [-6, 6, 4], [2, -4, 8], [12, -4, 0], [-2, -12, 4], [2, 8, -1], [-6, 2, 0], [-11, 1, 3], [-7, -3, -2], [-6, -4, 2], [-6, 6, -6], [-1, -9, 1], [-5, -1, -7], [-3, -1, -3], [-6, 2, 2], [-8, 8, -8], [-7, 5, -2], [7, -5, -1], [-2, 12, -4], [-6, -2, -2], [6, -6, -1], [2, -10, 0], [-9, 9, -2], [-17, 3, 0], [-4, 2, 0], [-16, 2, -1], [-4, 8, 3], [-3, -1, 4], [8, -2, -5], [-17, -7, 4], [-2, -8, 1], [-14, 6, -6], [7, 3, 5], [3, 5, 6], [13, 3, -7], [-4, 0, 1], [3, -1, -6], [0, 8, -1], [-10, 8, -8], [-7, -5, 1], [-12, 6, -1], [12, -12, 5], [-14, 6, -4], [-5, 7, 8], [-3, -3, 7], [-3, -7, 3], [-2, -8, 2], [20, 0, -5], [9, -9, 1], [-2, 4, 2], [5, -1, 1], [10, -2, -9], [8, 4, 2], [24, 4, -4], [-28, 2, -4], [-22, -4, 2], [6, -12, 8], [-9, 9, 5], [8, 0, 6], [-5, -5, -10], [18, 0, 9], [-3, -9, 3], [-10, 6, -2], [13, -3, -10], [-15, 3, -8], [5, -17, 10], [2, 6, 8], [18, -12, 8], [-6, -14, 2], [11, -3, -10], [-3, 5, 8], [-5, 7, -13], [16, -4, -8], [-16, 2, 4], [-3, -3, -8], [23, -3, -4], [-7, 19, -3], [8, -2, 14], [-12, -8, -2], [-4, -2, 0], [25, -3, 3], [-10, 12, -9], [4, -2, -1], [-7, 11, -16], [-2, -2, 10], [9, -5, 9], [-11, -5, 8], [2, -6, 2], [-18, 8, 6], [8, 2, -10], [-7, 21, -1], [-6, -2, 8], [9, -5, 3], [-12, -6, 2], [12, -6, 18], [-5, 3, 2], [23, -3, 7], [-20, 4, -1], [-12, -14, -1], [4, -16, 6], [20, -8, -6], [-17, 7, -9], [-24, 10, -8], [20, 0, -1], [-13, 17, 2], [12, -6, -5], [-12, -10, -6], [8, 18, -6], [2, 4, 1], [-7, 5, 1], [6, 6, -12], [-17, 3, 14], [2, -14, 7], [22, -6, 2], [-20, 10, -8], [8, 10, -8], [16, -2, 7], [27, 5, 5], [-8, 0, 17], [34, 0, 2], [-19, -5, 1], [-5, 15, 0], [-2, 10, 8], [-21, 1, 7], [1, -17, 7], [-17, -11, 4], [-2, 4, -14], [-10, -12, 0], [-16, -8, 10], [7, -13, 11], [-16, -4, -6], [-39, 9, -8], [25, -9, 2], [5, -5, 15], [-6, -18, -2], [35, 3, 1], [4, -8, 9], [-4, -2, 14], [6, -14, 20], [22, -8, 6], [-2, 6, 0], [18, 18, -5], [-12, -4, 8], [-7, -21, 6], [7, 33, -13], [-32, 6, 6], [21, -9, -11], [-18, 12, -8], [-42, 8, -13], [0, 8, -10], [-18, 20, -14], [-6, -28, 8], [0, -4, 10], [-11, 17, -1], [-4, -10, 8], [2, -18, 0], [33, -11, 11], [-11, 1, 12], [-30, 6, -9], [10, -22, 10], [20, -4, 18], [-4, -26, 6], [-15, -17, 5], [2, -24, 10], [0, -22, 6], [5, -11, 11], [-2, 24, 4], [-1, 7, -2], [-34, -4, 10], [6, 24, -13], [-21, 15, 7], [-6, 4, -12], [-33, 7, 0], [-11, 7, -16], [4, -16, 10], [-5, -1, -22], [29, -11, -15], [28, 14, -11], [-19, 25, -13], [-31, 11, 8], [4, -16, 14], [-50, 6, 6], [7, -5, -8], [38, 6, -11], [11, -19, 13], [-1, 7, -5], [35, -1, 12], [-4, 10, -18], [-17, -9, -9], [16, -14, 19], [-7, 7, 18], [37, 23, -7], [-39, 19, 3], [-21, 3, 15], [-6, -4, 4], [-13, 5, 9], [22, 18, -2], [21, 3, 9], [10, -12, -4], [27, 1, -11], [6, 12, -7], [37, -11, 5], [17, 7, 5], [-38, 2, -12], [-3, 13, 11], [4, 2, 18], [24, -12, -20], [9, -7, 14], [10, 2, -6], [12, -16, 0], [-14, 2, 20], [26, 14, 7], [30, 12, -14], [-1, -39, 9], [8, -12, 18], [-7, 9, -28], [-4, -14, 7], [30, -18, -12], [-26, -18, 13], [-13, -5, 11], [-2, 22, -4], [-6, -6, -19], [36, -18, 22], [20, -28, 22], [-25, 15, -17], [-3, 35, -19], [-27, 3, 14], [-19, 1, -3], [-10, 28, -15], [-25, 19, -10], [-8, -6, 14], [-28, 26, 8], [-9, 19, 9], [-10, -28, 7], [33, -1, -12], [36, -6, 6], [38, -14, 24], [-27, 3, 0], [14, 0, -18], [-16, -2, -9], [-32, 20, -1], [16, -26, 21], [14, -16, 8], [4, -12, 4], [21, -11, -12], [-28, -2, 0], [-31, -7, -7], [-7, -3, 2], [-21, 15, -20], [-1, -7, -3], [20, 16, -18], [10, -12, 14], [30, -10, 10], [53, -7, 8], [-3, 13, 1], [12, 10, -22], [-9, 33, -11], [-33, 13, -16], [-23, -21, 2], [28, -20, -6], [-27, -21, 2], [39, -15, 3], [11, 19, -5], [53, 7, 0], [22, 18, -14], [8, -6, -26], [20, -24, 32], [5, -1, -1], [-2, 28, 6], [18, 36, -16], [40, -18, 5], [-19, -15, 6], [52, -6, 7], [50, 8, 4], [-39, 15, -3], [-12, -28, 11], [-62, -6, 0], [-11, 1, -23], [-33, -1, 6], [-8, -10, -22], [-16, 6, -8], [0, -30, 13], [1, 23, -9], [12, -2, 32], [-32, 16, 2], [-19, -1, -19], [-48, 6, -14], [41, 23, -6], [-56, -8, 10], [-30, 4, -14], [-19, -7, 11], [21, 11, -1], [35, -9, 7], [22, 0, -12], [47, 5, -15], [77, -9, -5], [-25, 15, -30], [-36, 10, -6], [-8, 26, -20], [14, 32, -8], [-44, 28, 4], [-34, 6, 6], [50, 16, -1], [-3, 9, -22], [3, -1, 9], [3, -25, 13], [15, -29, 1], [22, 16, 7], [18, -32, 14], [-10, 16, -2], [-42, -8, 2], [6, 14, 0], [18, -32, 2], [36, -8, 12], [38, 8, 10], [-42, -20, 11], [-4, 4, -24], [-76, 10, -4], [-45, -7, 5], [-18, 18, 17], [-41, 1, 2], [-68, 10, 1], [-8, 2, 4], [14, -20, -18], [31, -7, 5], [36, 0, 0], [-15, -11, -17], [-3, 7, -19], [-12, 4, 25], [34, -28, 12], [5, 37, 2], [6, -34, 2], [6, 2, -4], [31, -9, -3], [28, -26, 22], [6, 12, -27], [-24, 4, 9], [-32, 20, -16], [28, 16, -8], [38, 2, 14], [19, 1, -12], [-10, -10, 24], [-23, 5, 13], [29, -29, 13], [13, -37, 15], [-10, 28, -18], [-47, -5, 6], [17, 5, 5], [-43, 11, -18], [29, -19, 21], [-18, 4, 13], [-30, 28, -14], [-30, -2, 12], [-16, -12, -6], [43, -19, -19], [20, -30, 8], [-26, 0, 1], [36, -20, -16], [20, 14, -18], [5, -9, -12], [-14, -14, 18], [-12, 8, 16], [-12, 4, -4], [-22, 4, -10], [31, 21, -13], [-4, 18, 14], [-18, 8, 9], [4, 34, -14], [8, -40, 27], [-24, 10, 2], [34, 8, -23], [-20, 2, -26], [5, 13, 1], [37, -9, -19], [26, 2, -3], [-3, 15, -16], [29, -5, 5], [-3, 19, -19], [14, 2, -1], [14, -4, -2], [-14, 10, 6], [42, 18, -6], [-51, -11, 9], [-15, -31, -7], [7, -15, -19], [51, 1, 11], [49, -21, -18], [48, 20, -14], [-19, -23, 13], [6, -4, -24], [-55, -9, 7], [3, 43, -16], [36, -16, 3], [-75, 13, -19], [-43, 21, 0], [-11, -13, 22], [-57, 9, -13], [13, 25, -16]]
